const fs = require("fs");
const path = require("path");

module.exports = {
	isExists(url) {
		return fs.existsSync(url);
	},
	getDir(url) {
		return fs.readdirSync(url);
	},
	isDirectory(url) {
		return fs.statSync(url).isDirectory();
	},
	forEachFolder(url, callback) {
		if (this.isExists(url)) {
			callback("folderBefore", url);
			const files = this.getDir(url);
			files.forEach((file) => {
				const curUrl = path.join(url, file);
				if (this.isDirectory(curUrl)) {
					this.forEachFolder(curUrl, callback);
				} else {
					callback("file", curUrl, file);
				}
			});
			callback("folderAfter", url);
		}
	},
	deleteFile(url) {
		fs.unlinkSync(url);
	},
	deleteFolder(url) {
		fs.rmdirSync(url);
	},
	deleteCatalog(url) {
		this.forEachFolder(url, (type, urlname, filename) => {
			if (type === "file") {
				this.deleteFile(urlname);
			}
			if (type === "folderAfter") {
				this.deleteFolder(urlname);
			}
		});
	},
	delete(url) {
		if (this.isExists(url)) {
			if (this.isDirectory(url)) {
				this.deleteCatalog(url);
			} else {
				this.deleteFile(url);
			}
		}
	},
	copyFile(src, tar) {
		if (this.isExists(src)) {
			fs.copyFileSync(src, tar);
		}
	},
	copyCatalog(src, tar) {
		if (this.isExists(src)) {
			if (!this.isExists(tar)) {
				this.createFolder(tar);
			}
			const files = this.getDir(src);
			files.forEach((file) => {
				const srcUrl = path.join(src, file);
				const tarUrl = path.join(tar, file);
				if (this.isDirectory(srcUrl)) {
					this.copyCatalog(srcUrl, tarUrl);
				} else {
					this.copyFile(srcUrl, tarUrl);
				}
			});
		}
	},
	copy(src, tar) {
		if (this.isExists(src)) {
			if (this.isDirectory(src)) {
				this.copyCatalog(src, tar);
			} else {
				this.copyFile(src, tar);
			}
		}
	},
	createFolder(url) {
		fs.mkdirSync(url);
	},
	createFile(url, data) {
		fs.writeFileSync(url, data);
	},
};
