const express = require("express");
const chalk = require("chalk");
const open = require("open");
const { createProxyMiddleware } = require("http-proxy-middleware");
const { exec } = require("child_process");
const pathUtil = require("path");
const {
  port,
  devServer: { proxy, serve, build },
} = require("../server.config");
const systemMap = require("../system");
const app = express();

if (build && build.length) {
  console.log(chalk.yellow(`异步构建子系统${build.join(",")}`));
  exec(`npm run build ${build.join(" ")}`, {
    maxBuffer: 1024 * 1024 * 1024,
  });
}

if (proxy) {
  if (Array.isArray(proxy)) {
    proxy.forEach((pro) => {
      pro.context &&
        pro.context.forEach((path) => {
          console.log(
            chalk.green(`开启API代理路径：${path}，目标：${pro.target}`)
          );
          app.use(path, createProxyMiddleware(pro));
        });
    });
  } else {
    Object.keys(proxy).forEach((path) => {
      console.log(
        chalk.green(`开启API代理路径：${path}，目标：${proxy[path].target}`)
      );
      app.use(path, createProxyMiddleware(proxy[path]));
    });
  }
}

Object.keys(systemMap).forEach((item) => {
  const { port, path, resource } = systemMap[item];
  const stationary = systemMap[item].static;
  if (serve && serve.includes(item)) {
    console.log(
      chalk.green(
        `${item}应用启动成功[子进程热启动]，执行命令：cd ${resource} && npm start`
      )
    );
    exec(`cd ${resource} && npm start`, {
      maxBuffer: 1024 * 1024 * 1024,
    });
    app.use(
      path,
      createProxyMiddleware({
        target: `http://localhost:${port}`,
      })
    );
  } else {
    app.use(path, express.static(stationary));
    console.log(
      chalk.green(`${item}应用启动成功[本地资源]，资源路径：${stationary}`)
    );
  }
});

app.listen(port || 3000);

// open(`http://localhost:${port || 3000}`);
